#!/bin/sh

APPOPTICS_TOKEN=${APPOPTICS_TOKEN:-"APPOPTICS_TOKEN"}
APPOPTICS_URL=${APPOPTICS_URL:-"https://metrics-api.librato.com/v1/measurements"}
DEFAULT_INSTALL_DIR=/opt/appoptics
INSTALL_DIR=${INSTALL_DIR:-${DEFAULT_INSTALL_DIR}}
APPOPTICS_CONF="${INSTALL_DIR}/etc/config.yaml"
SNAP_SERVICE_NAME=appoptics-snapteld
AGENT_DISPLAY_NAME="AppOptics Host Agent"
SNAP_TMP_DIR=/tmp/appoptics-snaptel
SNAP_LOG_DIR=/var/log/appoptics

SYSTEMCTL_BIN=`command -v systemctl 2>/dev/null`
CHKCONFIG_BIN=`command -v chkconfig 2>/dev/null`
SERVICE_BIN=`command -v service 2>/dev/null`

# Settings for the user used to run the daemon
APPOPTICS_USERNAME=appoptics
APPOPTICS_GROUPNAME=appoptics

# truth values
__IS_TRUE=1
__IS_FALSE=0

# Contains the current installation step of the post installer
__step=1

_ECHO_DEBUG=${_ECHO_DEBUG:-${__IS_FALSE}}

ROOT_DIR=`pwd`

# Return error codes
readonly SUCCESS=0
readonly FAILURE=1
readonly FAILURE_SERVICE_NOT_RUNNING=2
readonly FAILURE_NOT_ENOUGH_RIGHTS=3
readonly FAILURE_UNABLE_TO_EXTRACT=4
readonly FAILURE_MISSING_EXECUTABLE=5
readonly FAILURE_CREATE_APPOPTICS_GROUP=6
readonly FAILURE_CREATE_APPOPTICS_USER=7
readonly FAILURE_SET_ACCESS_RIGHTS_APPOPTICS_USER=8

__increment_step()
{
   __step=$((__step+1))
}

_COLORS=${BS_COLORS:-$(tput colors 2>/dev/null || echo 0)}
detect_color_support() 
{
    if [ $? -eq 0 ] && [ "$_COLORS" -gt 2 ] ; then
        RC="\033[1;31m"
        GC="\033[1;32m"
        BC="\033[1;34m"
        YC="\033[1;33m"
        EC="\033[0m"
    else
        RC=""
        GC=""
        BC=""
        YC=""
        EC=""
    fi
}
detect_color_support 

# Override built-in echo for consistency. Has newline at end
echo () (
fmt=%s end=\\n IFS=" "

while [ $# -gt 1 ] ; do
    case "$1" in
        [!-]*|-*[!ne]*) break ;;
        *ne*|*en*) fmt=%b end= ;;
        *n*) end= ;;
        *e*) fmt=%b ;;
    esac
    shift
done

printf "$fmt$end" "$*"
)

echored() {
    printf "${RC}%s${EC}\n" "$*";
}

echogreen() {
    printf "${GC}%s${EC}\n" "$*";
}

echoblue() {
    printf "${BC}%s${EC}\n" "$*";
}

echoyellow() {
    printf "${YC}%s${EC}\n" "$*";
}

exists()
{
  command -v "$1" >/dev/null 2>&1
}

now()
{
  echo $(date +'%Y-%m-%d-%H_%M_%S')
}

echoerror() {
    printf "${RC} * ERROR${EC}: %s\n" "$*" 1>&2;
}

info ()
{
    printf "${GC} *  INFO${EC}: %s\n" "$*";
}

warning ()
{
  printf "${YC} *  WARN${EC}: %s\n" "$*";
}

echodebug()
{
    if [ "$_ECHO_DEBUG" = "$__IS_TRUE" ]; then
        printf "${BC} * DEBUG${EC}: %s\n" "$*";
    fi
}

error ()
{
  error_code=${1}
  error_description=${2:-""}
  error_date_time="`echo $(date +'%Y-%m-%d at %H:%M:%S')`"
  echo -e "${BC}===============${RC} ERROR ${BC}======================${EC}"
  echo -e "${BC}An error occurred during installation of ${YC}${AGENT_DISPLAY_NAME}${BC}.${EC}"
  echo ""
  echo -e "${BC}Description: ${RC}${error_description}${BC}.${EC}"
  echo -e "${BC}Error code : ${RC}${error_code}${BC}.${EC}"
  echo -e "${BC}When       : ${RC}${error_date_time}${BC}.${EC}"
  echo ""
  echo -e "${BC}Please contact AppOptics customer support at ${YC}support@appoptics.com${BC}.${EC}"
  echo -e "${BC}========================================================${EC}"
}

display_warning_no_credentials ()
{
  warning "Environment variable APPOPTICS_TOKEN has not been set."
}

insert_credentials ()
{
    if [ "${APPOPTICS_TOKEN}" = "APPOPTICS_TOKEN" ] ; then
        display_warning_no_credentials
    else
        echodebug "Adding credentials ${APPOPTICS_TOKEN} into ${APPOPTICS_CONF} configuration file ..."
        sed -i "s/APPOPTICS_TOKEN/\"${APPOPTICS_TOKEN}\"/" ${APPOPTICS_CONF}
        sed -i "s,APPOPTICS_URL,\"${APPOPTICS_URL}\"," ${APPOPTICS_CONF}
        echodebug "Credentials added to ${APPOPTICS_CONF} file."
    fi
}

create_install_directories()
{
    echodebug "Creating support directories ..."
    mkdir -p ${SNAP_TMP_DIR}
    mkdir -p ${SNAP_LOG_DIR}
    echodebug "directories created."
}

welcome ()
{
  echo -e  " ${BC}================== ${GC} Congratulations ! ${BC}=======================${EC}"
  echo -e "${BC}The ${GC}${AGENT_DISPLAY_NAME}${BC} is now installed.${EC}"
  if [ "${APPOPTICS_TOKEN}" = "APPOPTICS_TOKEN" ] ; then
     echo ""
     echo -e "${BC} Installation has encountered some issues and warnings have been issued. Please review the following warnings.${EC}"
     echo ""
     echo -e "${BC}Environment variable ${YC}APPOPTICS_TOKEN${BC} was not available during package installation.${EC}"
     echo -e "${BC}Please edit ${YC}${APPOPTICS_CONF}${BC}, replace ${YC}APPOPTICS_TOKEN${BC} with your token, and restart the service.${EC}"
     echo ""
  else
     echogreen "You should begin seeing metrics in your AppOptics account any moment now."
  fi
}

failed ()
{
  error_code=${1}
  echo -e "${BC}===============${RC} FAILED ${BC}======================${EC}"
  echo -e "${BC}An error occurred during installation of ${RC}${AGENT_DISPLAY_NAME}${BC}.${EC}"
  echo -e "${BC}Please contact AppOptics customer support at ${RC}support@appoptics.com${BC}.${EC}"
  echo -e "${BC}Error code: ${RC}${error_code}${BC}.${EC}"
  echo -e "${BC}========================================================${EC}"
}

running_in_docker()
{
    grep -q "docker" /proc/self/cgroup
    RETVAL=$?
    if [ $RETVAL -eq 0 ] ; then
        return 1
    fi
    return 0
}

install_service()
{
    chmod 755 ${INSTALL_DIR}/bin/*
    chmod 755 ${INSTALL_DIR}/sbin/*

    info "(${__step}) Checking if system uses systemd or SysV(/etc/init.d) to manage the services."
    __increment_step

    # Check if system uses systemd
    if [ -x "${SYSTEMCTL_BIN}" ]; then
        info "Detected that system uses systemd to manage the services."
        if [ ! -w  /lib/systemd/system ]; then
             RETVAL=${FAILURE_NOT_ENOUGH_RIGHTS}
             error ${RETVAL} "Not enough user rights. Please run the script as superuser or check the rights on /lib/systemd/system directory."
             return $RETVAL
        else
            echodebug "Copying service files and configuration required by systemd ..."
            # Install script for a system that supports systemd
            cp ${INSTALL_DIR}/support/${SNAP_SERVICE_NAME}.service /lib/systemd/system/${SNAP_SERVICE_NAME}.service

            # Copy file containing service environment variables
            if [ -d /etc/default ] ; then
                cp ${INSTALL_DIR}/support/snapteld.sysconfig /etc/default/${SNAP_SERVICE_NAME}
            else
                if [ -d /etc/sysconfig ] ; then
                  cp ${INSTALL_DIR}/support/snapteld.sysconfig /etc/sysconfig/${SNAP_SERVICE_NAME}
                else
                  warning "Can't copy the ${SNAP_SERVICE_NAME} enviroment file."
                fi 
            fi

            # Reload daemon configurations
            info "(${__step}) Reloading configuration for ${SNAP_SERVICE_NAME} service ..."
            __increment_step

            ${SYSTEMCTL_BIN} daemon-reload
            echodebug "${SNAP_SERVICE_NAME} service reloaded."

            # Enable service to be executed at boot time
            info "(${__step}) Enabling ${SNAP_SERVICE_NAME} service to be loaded at boot time ..."
            __increment_step
            ${SYSTEMCTL_BIN} enable ${SNAP_SERVICE_NAME}.service

            # Check if service is enabled
            if [ "`${SYSTEMCTL_BIN} is-enabled ${SNAP_SERVICE_NAME}`" = "enabled" ]; then
               echodebug "Service ${SNAP_SERVICE_NAME} has been enabled to load at boot time."
            else
               warning "Unable to enable service ${SNAP_SERVICE_NAME}."
            fi

            # Check to see if running inside a docker with systemd
            can_start_service=${__IS_TRUE}
            running_in_docker
            RETVAL=$?
            if [ $RETVAL -eq 1 ] ; then
                echodebug "Running inside a Docker container with systemd."
                can_start_service=${__IS_FALSE}
            fi

            RETVAL=${SUCCESS}

            if [ ${can_start_service} -eq ${__IS_TRUE} ] ; then
                # Start service
                info "(${__step}) Starting ${SNAP_SERVICE_NAME} service ..."
                __increment_step
                ${SYSTEMCTL_BIN} start ${SNAP_SERVICE_NAME}.service
                RETVAL=$?
                if [ ${RETVAL} -eq ${SUCCESS} ]; then
                   echodebug "${SNAP_SERVICE_NAME} service has STARTED."
                else
                   RETVAL=${FAILURE_SERVICE_NOT_RUNNING}
                   error ${RETVAL} "Unable to start ${SNAP_SERVICE_NAME} service. Service is NOT STARTED."
                fi    

                # Wait for 10 seconds for service to be fully initialized.
                # Avoids case when service is running for a couple of seconds and fails because of configuration
                sleep 10

                # Check if service is running
                info "(${__step}) Checking if ${SNAP_SERVICE_NAME} service is running ..."
                __increment_step
                if [ "`${SYSTEMCTL_BIN} is-active ${SNAP_SERVICE_NAME}`" = "active" ]; then
                   echodebug "${SNAP_SERVICE_NAME} service is RUNNING."
                   RETVAL=${SUCCESS}
                else
                   RETVAL=${FAILURE_SERVICE_NOT_RUNNING}
                   error ${RETVAL} "Unable to start ${SNAP_SERVICE_NAME} service. Service is NOT RUNNING."
                fi
            fi

            return $RETVAL
       fi
    else
      # System uses SysV(/etc/init.d)
      info "Detected that system uses SysV(/etc/init.d) to manage the services."
      if [ ! -w /etc/init.d ]; then
         RETVAL=${FAILURE_NOT_ENOUGH_RIGHTS}
         error ${RETVAL} "Not enough user rights. Please run the script as superuser or check the rights on /etc/init.d directory."
         return $RETVAL
      else
           echodebug "Copying service files and configuration required by SysV ..."
           AWS=`grep -q Amazon /etc/issue`
           RETVAL=$?
           if [ $RETVAL = 0 -o -f /etc/redhat-release ]; then
               # Copy file containing service environment variables
               if [ -d /etc/sysconfig ] ; then
                  cp ${INSTALL_DIR}/support/snapteld.sysconfig /etc/sysconfig/${SNAP_SERVICE_NAME}
               else
                  warning "Can't copy the ${SNAP_SERVICE_NAME} enviroment file."
               fi

               # Install script for a RedHat based system
               cp ${INSTALL_DIR}/support/snapteld.rh.initd /etc/init.d/${SNAP_SERVICE_NAME}
               chmod +x /etc/init.d/${SNAP_SERVICE_NAME}
           else
               # Copy file containing service environment variables
               if [ -d /etc/default ] ; then
                  cp ${INSTALL_DIR}/support/snapteld.sysconfig /etc/default/${SNAP_SERVICE_NAME}
               else
                  warning "Can't copy the ${SNAP_SERVICE_NAME} enviroment file."
               fi

               # Install script for a Debian based system
               cp ${INSTALL_DIR}/support/snapteld.deb.initd /etc/init.d/${SNAP_SERVICE_NAME}
               chmod +x /etc/init.d/${SNAP_SERVICE_NAME}
           fi

           info "(${__step}) Enabling ${SNAP_SERVICE_NAME} service to be loaded at boot time..."
           __increment_step
           # Enable service to be executed at boot time
           if [ -x "${CHKCONFIG_BIN}" ]; then
               ${CHKCONFIG_BIN} ${SNAP_SERVICE_NAME} on
           else
               /usr/sbin/update-rc.d ${SNAP_SERVICE_NAME} defaults
           fi
           info "${SNAP_SERVICE_NAME} service ENABLED to load at boot time."

           info "(${__step}) Starting ${SNAP_SERVICE_NAME} service ..."
           __increment_step
           # Start service
           if exists ${SERVICE_BIN} ; then
               ${SERVICE_BIN} ${SNAP_SERVICE_NAME} start
               RETVAL=$?
               if [ ${RETVAL} -eq ${SUCCESS} ] ; then
                   echodebug "${SNAP_SERVICE_NAME} service is STARTED."
               else
                   RETVAL=${FAILURE_SERVICE_NOT_RUNNING}
                   error ${RETVAL} "Unable to start ${SNAP_SERVICE_NAME} service. Service is NOT RUNNING."
               fi
           else
               RETVAL=${FAILURE_MISSING_EXECUTABLE}
               error ${RETVAL} "Unable to start ${SNAP_SERVICE_NAME} service. Service ${SERVICE_BIN} command is missing."
           fi
              
           return $RETVAL
      fi 
    fi
}

create_daemon_user()
{
    info "(${__step}) Configuring user and group settings ..."
    __increment_step

    # Check if the group already exists
    if getent group ${APPOPTICS_GROUPNAME} >/dev/null; then
        info "Group ${APPOPTICS_GROUPNAME} already exists. No need to create it."
    else
       info "Creating ${APPOPTICS_GROUPNAME} system group ..."
       groupadd --system ${APPOPTICS_GROUPNAME}
       RETVAL=$?
       if [ $RETVAL -ne 0 ]; then
           echoerror "Unable to create ${APPOPTICS_GROUPNAME} system group. Error code: ${RETVAL}."
           return ${FAILURE_CREATE_APPOPTICS_GROUP}
       fi
       info "${APPOPTICS_GROUPNAME} group created."
    fi

   if ! getent passwd ${APPOPTICS_USERNAME} >/dev/null; then
       info "Creating ${APPOPTICS_USERNAME} system user ..."
       useradd --system \
               --gid ${APPOPTICS_GROUPNAME} \
               --no-create-home \
               --home-dir ${INSTALL_DIR} \
               --comment "${AGENT_DISPLAY_NAME} Daemon" \
               --shell /bin/sh \
               ${APPOPTICS_USERNAME}
       RETVAL=$?
       if [ $RETVAL -ne 0 ]; then
          echoerror "Unable to create ${APPOPTICS_USERNAME} system user. Error code: ${RETVAL}."
          return ${FAILURE_CREATE_APPOPTICS_USER}
       fi
       info "${APPOPTICS_USERNAME} system user created."
   else 
       info "System user ${APPOPTICS_USERNAME} already exists. No need to create it."
   fi 

   echodebug "user and group settings configured."
   return ${SUCCESS}
}

set_daemon_user_rights()
{   
    info "(${__step}) Setting user access rights ..."
    __increment_step

    ############################
    # General user requirements

    # Changed user to be owner of the installation directory
    chown ${APPOPTICS_USERNAME}:${APPOPTICS_GROUPNAME} -R ${INSTALL_DIR}

    # Changed user to be owner of the temporary directory.
    # Temporary directory is used by daemon to launch executables,
    # logging.
    chown ${APPOPTICS_USERNAME}:${APPOPTICS_GROUPNAME} -R ${SNAP_TMP_DIR}
    chown ${APPOPTICS_USERNAME}:${APPOPTICS_GROUPNAME} -R ${SNAP_LOG_DIR}
    chmod 777 -R ${SNAP_TMP_DIR}
    chmod 755 -R ${SNAP_LOG_DIR}

    ###########################
    # Docker user requirements

    # Add user to Docker group
    local docker_group_name=docker
    if getent group ${docker_group_name} >/dev/null; then
        info "Found ${docker_group_name} group. Adding ${APPOPTICS_USERNAME} to ${docker_group_name} group ..."
        usermod -aG ${docker_group_name} ${APPOPTICS_USERNAME}
        RETVAL=$?
        if [ $RETVAL -ne 0 ]; then
          echoerror "Unable to add ${APPOPTICS_USERNAME} user to ${docker_group_name} group. Error code: ${RETVAL}."
          return ${FAILURE_SET_ACCESS_RIGHTS_APPOPTICS_USER}
        else
          info "${APPOPTICS_USERNAME} added to ${docker_group_name} group."
        fi
    else
        warning "Unable to find ${docker_group_name} group. Docker plugin will not work without ${APPOPTICS_USERNAME} user being added to ${docker_group_name} group."
    fi

    echodebug "user access rights configured."
    return ${SUCCESS}
}

main()
{
    echo -e  " ${BC}======================${GC} ${AGENT_DISPLAY_NAME} Post Install ${BC}======================="
    echoblue "  This script will complete post installation of ${AGENT_DISPLAY_NAME}."

    create_install_directories
    insert_credentials
   
    create_daemon_user
    RETVAL=$?
    if [ ${RETVAL} -ne ${SUCCESS} ] ; then
        echo -e  "${BC}Post installation of ${GC}${AGENT_DISPLAY_NAME}${BC} has ${RC}FAILED${BC}. Unable to create ${APPOPTICS_USERNAME} system user.${EC}"
        failed ${RETVAL}
        exit ${RETVAL}
    fi

    set_daemon_user_rights
    RETVAL=$?
    if [ ${RETVAL} -ne ${SUCCESS} ] ; then
        echo -e  "${BC}Post installation of ${GC}${AGENT_DISPLAY_NAME}${BC} has ${RC}FAILED${BC}. Unable to set access rights for ${APPOPTICS_USERNAME} system user.${EC}"
        failed ${RETVAL}
        exit ${RETVAL}
    fi

    install_service
    RETVAL=$?
    if [ ${RETVAL} -eq ${FAILURE_SERVICE_NOT_RUNNING} ] ; then
        RETVAL=${SUCCESS}
        echo -e  "${BC}Post installation of ${GC}${AGENT_DISPLAY_NAME}${BC} complete, but the agent did not start. Please contact support if this is unexpected."
        echo -e  "${BC}======================${GC} ${AGENT_DISPLAY_NAME} Post Install Completed ${BC}======================="
    elif [ ${RETVAL} -ne ${SUCCESS} ] ; then
        echo -e  "${BC}Post installation of ${GC}${AGENT_DISPLAY_NAME}${BC} has ${RC}FAILED${BC}. Unable to install service.${EC}"
        failed ${RETVAL}
        echo -e  "${BC}======================${GC} ${AGENT_DISPLAY_NAME} Post Install Completed ${BC}======================="
        exit ${RETVAL}
    else
        echo -e  "${BC}Post installation of ${GC}${AGENT_DISPLAY_NAME}${BC} has ${GC}SUCCEEDED${BC}.${EC}"
        echo -e  "${BC}======================${GC} ${AGENT_DISPLAY_NAME} Post Install Completed ${BC}======================="
        welcome
    fi

    exit ${SUCCES}
}

main

