#!/bin/sh

DEFAULT_INSTALL_DIR=/opt/appoptics
INSTALL_DIR=${DEFAULT_INSTALL_DIR}
APPOPTICS_CONF="${INSTALL_DIR}/etc/config.yaml"
SNAP_SERVICE_NAME=appoptics-snapteld
AGENT_DISPLAY_NAME="AppOptics Host Agent"
SNAP_TMP_DIR=/tmp/appoptics-snaptel

SYSTEMCTL_BIN=`command -v systemctl 2>/dev/null`
CHKCONFIG_BIN=`command -v chkconfig 2>/dev/null`
SERVICE_BIN=`command -v service 2>/dev/null`

# Settings for the user used to run the daemon
APPOPTICS_USERNAME=appoptics
APPOPTICS_GROUPNAME=appoptics

# truth values
__IS_TRUE=1
__IS_FALSE=0

_ECHO_DEBUG=${_ECHO_DEBUG:-${__IS_FALSE}}

# Contains the current installation step of the post installer
__step=1


# Return error codes
readonly SUCCESS=0
readonly FAILURE=1
readonly FAILURE_SERVICE_NOT_RUNNING=2
readonly FAILURE_NOT_ENOUGH_RIGHTS=3
readonly FAILURE_UNABLE_TO_EXTRACT=4
readonly FAILURE_MISSING_EXECUTABLE=5

__increment_step()
{
   __step=$((__step+1))
}

_COLORS=${BS_COLORS:-$(tput colors 2>/dev/null || echo 0)}
detect_color_support() 
{
    if [ $? -eq 0 ] && [ "$_COLORS" -gt 2 ] ; then
        RC="\033[1;31m"
        GC="\033[1;32m"
        BC="\033[1;34m"
        YC="\033[1;33m"
        EC="\033[0m"
    else
        RC=""
        GC=""
        BC=""
        YC=""
        EC=""
    fi
}
detect_color_support 

# Override built-in echo for consistency. Has newline at end
echo () (
fmt=%s end=\\n IFS=" "

while [ $# -gt 1 ] ; do
    case "$1" in
        [!-]*|-*[!ne]*) break ;;
        *ne*|*en*) fmt=%b end= ;;
        *n*) end= ;;
        *e*) fmt=%b ;;
    esac
    shift
done

printf "$fmt$end" "$*"
)

echored() {
    printf "${RC}%s${EC}\n" "$*";
}

echogreen() {
    printf "${GC}%s${EC}\n" "$*";
}

echoblue() {
    printf "${BC}%s${EC}\n" "$*";
}

echoyellow() {
    printf "${YC}%s${EC}\n" "$*";
}

exists()
{
  command -v "$1" >/dev/null 2>&1
}

now()
{
  echo $(date +'%Y-%m-%d-%H_%M_%S')
}

echoerror() {
    printf "${RC} * ERROR${EC}: %s\n" "$*" 1>&2;
}

info ()
{
    printf "${GC} *  INFO${EC}: %s\n" "$*";
}

warning ()
{
  printf "${YC} *  WARN${EC}: %s\n" "$*";
}

echodebug()
{
    if [ "$_ECHO_DEBUG" = "$__IS_TRUE" ]; then
        printf "${BC} * DEBUG${EC}: %s\n" "$*";
    fi
}

error ()
{
  error_code=${1}
  error_description=${2:-""}
  error_date_time="`echo $(date +'%Y-%m-%d at %H:%M:%S')`"
  echoerror "######################################################"
  echoerror "An error has occurred during upgrade of the agent."
  echoerror
  echoerror "Description: ${error_description}"
  echoerror "Error code : ${error_code}"
  echoerror "When       : ${error_date_time}"
  echoerror "######################################################"
}

failed ()
{
  error_code=${1}
  echored "###################################################"
  echored "An error occurred during the installation of ${AGENT_DISPLAY_NAME}."
  echored "Please contact AppOptics customer service for support at support@appoptics.com."
  echored "###################################################"
}

create_daemon_user()
{
    info "(${__step}) Configuring user and group settings ..."
    __increment_step

    # Check if the group already exists
    if getent group ${APPOPTICS_GROUPNAME} >/dev/null; then
        info "Group ${APPOPTICS_GROUPNAME} already exists. No need to create it."
    else
       info "Creating ${APPOPTICS_GROUPNAME} system group ..."
       groupadd --system ${APPOPTICS_GROUPNAME}
       RETVAL=$?
       if [ $RETVAL -ne 0 ]; then
           echoerror "Unable to create ${APPOPTICS_GROUPNAME} system group. Error code: ${RETVAL}."
           return ${FAILURE_CREATE_APPOPTICS_GROUP}
       fi
       info "${APPOPTICS_GROUPNAME} group created."
    fi

   if ! getent passwd ${APPOPTICS_USERNAME} >/dev/null; then
       info "Creating ${APPOPTICS_USERNAME} system user ..."
       useradd --system \
               --gid ${APPOPTICS_GROUPNAME} \
               --no-create-home \
               --home-dir ${INSTALL_DIR} \
               --comment "${AGENT_DISPLAY_NAME} Daemon" \
               --shell /bin/sh \
               ${APPOPTICS_USERNAME}
       RETVAL=$?
       if [ $RETVAL -ne 0 ]; then
          echoerror "Unable to create ${APPOPTICS_USERNAME} system user. Error code: ${RETVAL}."
          return ${FAILURE_CREATE_APPOPTICS_USER}
       fi
       info "${APPOPTICS_USERNAME} system user created."
   else 
       info "System user ${APPOPTICS_USERNAME} already exists. No need to create it."
   fi 

   echodebug "user and group settings configured."
   return ${SUCCESS}
}

set_daemon_user_rights()
{   
    info "(${__step}) Setting user access rights ..."
    __increment_step

    ############################
    # General user requirements

    # Changed user to be owner of the installation directory
    chown ${APPOPTICS_USERNAME}:${APPOPTICS_GROUPNAME} -R ${INSTALL_DIR}

    # Changed user to be owner of the temporary directory.
    # Temporary directory is used by daemon to launch executables,
    # logging.
    chown ${APPOPTICS_USERNAME}:${APPOPTICS_GROUPNAME} -R ${SNAP_TMP_DIR}
    chmod 777 -R ${SNAP_TMP_DIR}

    ###########################
    # Docker user requirements

    # Add user to Docker group
    local docker_group_name=docker
    if getent group ${docker_group_name} >/dev/null; then
        info "Found ${docker_group_name} group. Adding ${APPOPTICS_USERNAME} to ${docker_group_name} group ..."
        usermod -aG ${docker_group_name} ${APPOPTICS_USERNAME}
        RETVAL=$?
        if [ $RETVAL -ne 0 ]; then
          echoerror "Unable to add ${APPOPTICS_USERNAME} user to ${docker_group_name} group. Error code: ${RETVAL}."
          return ${FAILURE_SET_ACCESS_RIGHTS_APPOPTICS_USER}
        else
          info "${APPOPTICS_USERNAME} added to ${docker_group_name} group."
        fi
    else
        warning "Unable to find ${docker_group_name} group. Docker plugin will not work without ${APPOPTICS_USERNAME} user being added to ${docker_group_name} group."
    fi

    echodebug "user access rights configured."
    return ${SUCCESS}
}

running_in_docker()
{
    grep -q "docker" /proc/self/cgroup
    RETVAL=$?
    if [ $RETVAL -eq 0 ] ; then
        return 1
    fi
    return 0
}

start_service()
{
    info "(${__step}) Checking if system uses systemd or SysV(/etc/init.d) to manage the services."
    __increment_step

    if [ -x "${SYSTEMCTL_BIN}" ]; then
       # Check to see if running inside a docker with systemd
        can_start_service=${__IS_TRUE}
        running_in_docker
        RETVAL=$?
        if [ $RETVAL -eq 1 ] ; then
            echodebug "Running inside a Docker container with systemd."
            can_start_service=${__IS_FALSE}
        fi

        RETVAL=${SUCCESS}

        if [ ${can_start_service} -eq ${__IS_TRUE} ] ; then
            # Start service
            info "(${__step}) Starting ${AGENT_DISPLAY_NAME} service ..."
            __increment_step
            ${SYSTEMCTL_BIN} start ${SNAP_SERVICE_NAME}.service
            RETVAL=$?
            if [ ${RETVAL} -eq ${SUCCESS} ]; then
                echodebug "${SNAP_SERVICE_NAME} service has STARTED."
            else
                RETVAL=${FAILURE_SERVICE_NOT_RUNNING}
                error ${RETVAL} "Unable to start ${SNAP_SERVICE_NAME} service. Service is NOT STARTED."
            fi    

            sleep 10

            info "(${__step}) Checking if ${SNAP_SERVICE_NAME} service is running ..."
            __increment_step
            if [ "`${SYSTEMCTL_BIN} is-active ${SNAP_SERVICE_NAME}`" = "active" ]; then
                echodebug "${SNAP_SERVICE_NAME} service is RUNNING."
                RETVAL=${SUCCESS}
            else
                RETVAL=${FAILURE_SERVICE_NOT_RUNNING}
                error ${RETVAL} "Unable to start ${SNAP_SERVICE_NAME} service. Service is NOT RUNNING."
            fi
        fi

        return $RETVAL

    else
        info "(${__step}) Starting ${SNAP_SERVICE_NAME} service ..."
        __increment_step
        # Start service
        if exists ${SERVICE_BIN} ; then
            ${SERVICE_BIN} ${SNAP_SERVICE_NAME} start
            RETVAL=$?
            if [ ${RETVAL} -eq ${SUCCESS} ] ; then
                echodebug "${SNAP_SERVICE_NAME} service is STARTED."
            else
                RETVAL=${FAILURE_SERVICE_NOT_RUNNING}
                error ${RETVAL} "Unable to start ${SNAP_SERVICE_NAME} service. Service is NOT RUNNING."
            fi
        else
            RETVAL=${FAILURE_MISSING_EXECUTABLE}
            error ${RETVAL} "Unable to start ${SNAP_SERVICE_NAME} service. Service ${SERVICE_BIN} command is missing."
        fi
        
        return $RETVAL
    fi
}

create_install_directories()
{
    echodebug "Creating support directories ..."
    mkdir -p ${SNAP_TMP_DIR}
    echodebug "directories created."
}

check_previous_config()
{
    if [ -e ${APPOPTICS_CONF} ] ; then
        info "Found previous ${APPOPTICS_CONF} configuration file."

        # Create backup copy of the configuration file in case gets overritten
        config_filename=$(basename "${APPOPTICS_CONF}")
        config_backup_filename="${HOME}/${config_filename}.backup.appoptics-snaptel.`now`"
        info "Create backup of ${APPOPTICS_CONF} configuration file to ${config_backup_filename}."
        cp ${APPOPTICS_CONF} ${config_backup_filename}

        # If token was set during installation, warn user if is not the same as previous token
        if [ -n "${APPOPTICS_TOKEN}" ] ; then
            old_token=$(grep token ${APPOPTICS_CONF} | awk '{gsub(/"/, "", $2); print $2}')
            if [ "${old_token}" != "${APPOPTICS_TOKEN}" ] ; then
                warning "${AGENT_DISPLAY_NAME} was previously installed with token ${old_token} which is different than ${APPOPTICS_TOKEN} provided with this installation."
                warning "Update ${APPOPTICS_CONF} configuration file with the correct token."
            fi
        fi
    fi
}

main()
{
    echo -e  " ${BC}======================${GC} ${AGENT_DISPLAY_NAME} Post Upgrade Install ${BC}=======================${EC}"
    echoblue "  This script will complete post upgrade installation of ${AGENT_DISPLAY_NAME}."

    create_install_directories
    check_previous_config

    create_daemon_user
    RETVAL=$?
    if [ ${RETVAL} -ne ${SUCCESS} ] ; then
        echo -e  "${BC}Post upgrade installation of ${GC}${AGENT_DISPLAY_NAME}${BC} has ${RC}FAILED${BC}. Unable to create ${APPOPTICS_USERNAME} system user.${EC}"
        failed ${RETVAL}
        exit ${RETVAL}
    fi

    set_daemon_user_rights
    RETVAL=$?
    if [ ${RETVAL} -ne ${SUCCESS} ] ; then
       echo -e  "${BC}Post upgrade installation of ${GC}${AGENT_DISPLAY_NAME}${BC} has ${RC}FAILED${BC}. Unable to set access rights for ${APPOPTICS_USERNAME} system user.${EC}"
       failed ${RETVAL}
       exit ${RETVAL}
    fi

    start_service
    RETVAL=$?
    if [ ${RETVAL} -ne ${SUCCESS} ] ; then
        echo -e  "${BC}Post upgrade installation of ${GC}${AGENT_DISPLAY_NAME}${BC} has ${RC}FAILED${BC}. Unable to start service.${EC}"
        failed ${RETVAL}
        exit ${RETVAL}
    fi

    echo -e  "${BC}Post upgrade installation of ${GC}${AGENT_DISPLAY_NAME}${BC} has ${GC}SUCCEEDED${EC}"
    
    echo -e  " ${BC}======================${GC} ${AGENT_DISPLAY_NAME} Post Upgrade Install Completed ${BC}=======================${EC}"
    exit ${SUCCESS}
}

main


