#!/bin/sh

DEFAULT_INSTALL_DIR=/opt/appoptics
INSTALL_DIR=${DEFAULT_INSTALL_DIR}
APPOPTICS_CONF="${INSTALL_DIR}/etc/config.yaml"
SNAP_SERVICE_NAME=appoptics-snapteld
AGENT_DISPLAY_NAME="AppOptics Host Agent"
SNAP_TMP_DIR=/tmp/appoptics-snaptel

SYSTEMCTL_BIN=`command -v systemctl 2>/dev/null`
CHKCONFIG_BIN=`command -v chkconfig 2>/dev/null`
SERVICE_BIN=`command -v service 2>/dev/null`
RM_CMD=/bin/rm

ROOT_DIR=`pwd`

# Settings for the user used to run the daemon
APPOPTICS_USERNAME=appoptics
APPOPTICS_GROUPNAME=appoptics

# truth values
__IS_TRUE=1
__IS_FALSE=0

# Contains the current installation step of the post installer
__step=1

_ECHO_DEBUG=${__IS_FALSE}

__increment_step()
{
   __step=$((__step+1))
}

_COLORS=${BS_COLORS:-$(tput colors 2>/dev/null || echo 0)}
detect_color_support() 
{
    if [ $? -eq 0 ] && [ "$_COLORS" -gt 2 ] ; then
        RC="\033[1;31m"
        GC="\033[1;32m"
        BC="\033[1;34m"
        YC="\033[1;33m"
        EC="\033[0m"
    else
        RC=""
        GC=""
        BC=""
        YC=""
        EC=""
    fi
}
detect_color_support 

# Override built-in echo for consistency. Has newline at end
echo () (
fmt=%s end=\\n IFS=" "

while [ $# -gt 1 ] ; do
    case "$1" in
        [!-]*|-*[!ne]*) break ;;
        *ne*|*en*) fmt=%b end= ;;
        *n*) end= ;;
        *e*) fmt=%b ;;
    esac
    shift
done

printf "$fmt$end" "$*"
)

echored() {
    printf "${RC}%s${EC}\n" "$*";
}

echogreen() {
    printf "${GC}%s${EC}\n" "$*";
}

echoblue() {
    printf "${BC}%s${EC}\n" "$*";
}

echoyellow() {
    printf "${YC}%s${EC}\n" "$*";
}

exists()
{
  command -v "$1" >/dev/null 2>&1
}

now()
{
  echo $(date +'%Y-%m-%d-%H_%M_%S')
}

echoerror() {
    printf "${RC} * ERROR${EC}: %s\n" "$*" 1>&2;
}

info ()
{
    printf "${GC} *  INFO${EC}: %s\n" "$*";
}

warning ()
{
  printf "${YC} *  WARN${EC}: %s\n" "$*";
}

echodebug()
{
    if [ "$_ECHO_DEBUG" = "$__IS_TRUE" ]; then
        printf "${BC} * DEBUG${EC}: %s\n" "$*";
    fi
}

error ()
{
  error_code=${1}
  error_description=${2:-""}
  error_date_time="`echo $(date +'%Y-%m-%d at %H:%M:%S')`"
  echoerror "######################################################"
  echoerror "An error has occured during uninstallation of the agent."
  echoerror
  echoerror "Description: ${error_description}"
  echoerror "Error code : ${error_code}"
  echoerror "When       : ${error_date_time}"
  echoerror "######################################################"
}

remove_install_directories()
{
    echodebug "Deleting support directories ..."
    ${RM_CMD} -rf ${SNAP_TMP_DIR}
    echodebug "directories removed."
}

remove_service()
{
    info "(${__step}) Checking if system uses systemd or SysV(/etc/init.d) to manage the services."
    __increment_step

    if [ -x "${SYSTEMCTL_BIN}" ]; then
        # Stop service
        info "(${__step}) Stopping ${AGENT_DISPLAY_NAME} service ..."
        __increment_step
        ${SYSTEMCTL_BIN} stop ${SNAP_SERVICE_NAME}

        # Disable service to be executed at boot time
        info "(${__step}) Disabling ${AGENT_DISPLAY_NAME} service to be loaded at boot time ..."
        __increment_step
        ${SYSTEMCTL_BIN} disable ${SNAP_SERVICE_NAME}

        # Remove service
        echodebug "Remove configuration for ${AGENT_DISPLAY_NAME} service ..."
        ${RM_CMD} -f /lib/systemd/system/${SNAP_SERVICE_NAME}.service
    else
       # Stop service
       info "(${__step}) Stopping ${AGENT_DISPLAY_NAME} service ..."
       __increment_step
       ${SERVICE_BIN} ${SNAP_SERVICE_NAME} stop

       # Disable service to be executed at boot time
       info "(${__step}) Disabling ${AGENT_DISPLAY_NAME} service to be loaded at boot time ..."
       __increment_step
       if [ -x "${CHKCONFIG_BIN}" ]; then
           ${CHKCONFIG_BIN} ${SNAP_SERVICE_NAME} off
       else
           /usr/sbin/update-rc.d -f ${SNAP_SERVICE_NAME} remove
       fi

       # Remove service
       echodebug "Remove configuration for ${AGENT_DISPLAY_NAME} service ..."
       ${RM_CMD} -f /etc/init.d/${SNAP_SERVICE_NAME}
    fi
}

remove_daemon_user()
{
   info "(${__step}) Removing user and group settings ..."
   __increment_step

   if getent passwd ${APPOPTICS_USERNAME} >/dev/null; then
        info "Deleting ${APPOPTICS_USERNAME} system user ..."
        userdel ${APPOPTICS_USERNAME}
        RETVAL=$?
        if [ $RETVAL -ne 0 ]; then
           warning "Unable to delete ${APPOPTICS_USERNAME} system user. Error code: ${RETVAL}."
        else
           info "${APPOPTICS_USERNAME} deleted."
        fi
    fi

    if getent group ${APPOPTICS_GROUPNAME} >/dev/null; then
        info "Deleting ${APPOPTICS_GROUPNAME} system group ..."
        groupdel ${APPOPTICS_GROUPNAME}
        RETVAL=$?
        if [ $RETVAL -ne 0 ]; then
           warning "Unable to delete ${APPOPTICS_GROUPNAME} system group. Error code: ${RETVAL}."
        else
           info "${APPOPTICS_GROUPNAME} deleted."
        fi
    fi
 
    echodebug "user and group settings removed."
}

main()
{
    echo -e  " ${BC}======================${GC} ${AGENT_DISPLAY_NAME} Uninstall ${BC}=======================${EC}"
    echoblue "  This script will complete uninstallation of ${AGENT_DISPLAY_NAME}."

    remove_service
    remove_install_directories
    remove_daemon_user

    echo -e  "${BC} Uninstall of ${GC}${AGENT_DISPLAY_NAME}${BC} has completed.${EC}"

    echo -e  " ${BC}======================${GC} ${AGENT_DISPLAY_NAME} Uninstall Completed ${BC}=======================${EC}"
}

main

